from django.contrib import admin 
from django.http import HttpResponseRedirect, HttpResponse 
from django.core.urlresolvers import reverse 
from django.core import serializers
from django.utils.translation import ugettext_lazy as _

def export_as_csv(modeladmin, request, queryset, export_type="csv"):
    selected = request.POST.getlist(admin.ACTION_CHECKBOX_NAME) 
    model_string = queryset.model._meta.module_name
    app_label = queryset.model._meta.app_label
    return HttpResponseRedirect("%s?ids=%s" % (reverse('generic_export', 
        kwargs={'app_label': app_label, 'model_string': model_string,
        'export_type': export_type}), ",".join(selected)))
export_as_csv.short_description = _("Export as CSV")

def export_as_tsv(modeladmin, request, queryset):
    return export_as_csv(modeladmin, request, queryset, export_type="tsv")
export_as_tsv.short_description = _("Export as TSV")

def export_as_json(modeladmin, request, queryset):
    # Based on Django docs
    response = HttpResponse(mimetype="text/javascript")
    serializers.serialize("json", queryset, stream=response, ensure_ascii=False, indent=4)
    return response
export_as_json.short_description = _("Export as JSON")

admin.site.add_action(export_as_csv, 'export_csv')
admin.site.add_action(export_as_tsv, 'export_tsv')
admin.site.add_action(export_as_json, 'export_json')
