#!/usr/bin/python
# -*- coding: utf-8 -*
from datetime import datetime
from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.core import serializers
from django.core.serializers.json import DateTimeAwareJSONEncoder
from django.core import serializers
from django.utils import simplejson
from brightway.apps.base.fields.automaticurlmodel import AutomaticURLModel

json_serializer = serializers.get_serializer("json")()

class AutopopulateModel(AutomaticURLModel):
    """
Abstract base class that automatically updates 'modified' and 'modifier' fields, 
and sets 'created' and 'creator' fields upon model instantiation.
    """
    created = models.DateTimeField(_('created'), editable=False, 
        auto_now_add=True)
    modified = models.DateTimeField(_('modified'), editable=False, 
        auto_now=True)
    creator = models.ForeignKey(User, related_name="created_%(class)s_set", 
        verbose_name=_('creator'))
    modifier = models.ForeignKey(User, related_name="modified_%(class)s_set", 
        verbose_name=_('modifier'))

    class Meta:
        abstract = True
        get_latest_by = 'created'

    def save(self, *args, **kwargs):
        if self.creator_id and not self.modifier_id:
            # Creator must be specified, or provided automatically by view
            self.modifier_id = self.creator_id
        super(AutopopulateModel, self).save(*args, **kwargs)

    def created_json(self):
        return simplejson.dumps(self.created, cls=DateTimeAwareJSONEncoder)

    def modified_json(self):
        return simplejson.dumps(self.modified, cls=DateTimeAwareJSONEncoder)
