#!/usr/bin/python
"""
custom script that calls pylint with a special set of parameters.
Mostly borrowed from Turbogears: MIT license, with some help from 
django-extensions: new BSD license.
"""
import sys
import os
from django.core.management.base import BaseCommand, CommandError

class Command(BaseCommand):
    help = ("Run pylint on specified file.")

    requires_model_validation = False
    can_import_settings = True

    def handle(self, *args, **kwargs):
        if len(args) != 1:
            raise CommandError("Must specify eaxctly 1 file to check.")
        self.linty(args[0])

    def linty(self, filename):
        try:
            from pylint import lint
        except ImportError:
            print "Can't import module pylint. Did you install it?"
            sys.exit(-1)
        import webbrowser
        
        # A list of messages that should not be printed by pylint. 
        SUPRESSED_MESSAGES = [
            'I0011', # Used when an inline option disable a message or a messages 
                     # category.
        # If you decided to globally switch of a certain message instead of doing so
        # in file or scope where its generated then you can just uncomment it here.
        # Or add it if its not in the list.
        #   'E1101', # Used when a class is accessed for an unexistant member.
        #   'E0602', # Used when an undefined variable is accessed.
        #   'W0232', # Used when a class has no __init__ method, neither its parent 
        #            # classes.
        #   'W0401', # Used when `from module import *` is detected.
        #   'W0611', # Used when an imported module or variable is not used.
        #   'R0201', # Used when a method doesn't use its bound instance, and so could 
        #            # be written as a function.
        #   'R0801', # Indicates that a set of similar lines has been detected among 
        #            # multiple file.
        # Other messages worth ignoring (sometimes):
        # From: http://stackoverflow.com/questions/35470/are-there-any-static-analysis-tools-for-python/35478
        #
        # Brain-dead errors regarding standard language features
            'W0142', # *args and **kwargs support
        #   W0403 = Relative imports

        # Pointless whinging
        #   R0201 = Method could be a function
        #   W0212 = Accessing protected attribute of client class
        #   W0613 = Unused argument
        #   W0232 = Class has no __init__ method
        #   R0903 = Too few public methods
        #   C0301 = Line too long
        #   R0913 = Too many arguments
        #   C0103 = Invalid name
        #   R0914 = Too many local variables

        # PyLint's module importation is unreliable
        #   F0401 = Unable to import module
        #   W0402 = Uses of a deprecated module

        # Already an error when wildcard imports are used
        #   W0614 = Unused import from wildcard

        # Sometimes disabled depending on how bad a module is
        #   C0111 = Missing docstring
        ]

        PARAMS = [
            '--reports=y', '--include-ids=y', '-fcolorized', #'-fhtml',
                  '--disable=%s' % ",".join(SUPRESSED_MESSAGES), 
        ]
        PARAMS.extend((filename,))
        # Commented out because HTML output doesn't seem to work...
        # tempfile = open("pylint_output.html", "w")
        # saveout = sys.stdout
        # sys.stdout = tempfile
        lint.Run(PARAMS)
        # sys.stdout = saveout
        # tempfile.close()
        # webbrowser.open("pylint_output.html")