from django.conf import settings
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django import forms
from brightway.apps.group_permissions.admin import GroupPermissionAdminInline
from brightway.apps.geo.models import Geography, GeographyDatabase, \
    GeographySimplified


class GeographyDatabaseAdmin(admin.ModelAdmin):
    save_on_top = True
    search_fields = ('database',)
    list_display = ('database', 'description', 'geography_count')
    list_filter = ('license',)
    list_display_links = ('database',)
    ordering = ('database',)
    fieldsets = (
        (None, {
            'fields': ('database', 'description', 'license'),
            }
        ),
    )
    inlines = [GroupPermissionAdminInline,]

admin.site.register(GeographyDatabase, GeographyDatabaseAdmin)

class DatabaseAdmin(admin.ModelAdmin):
    save_on_top = True
    search_fields = ('database',)
    list_display = ('database', 'description', 'category_count', 
        'process_count', 'creator', 'created')
    list_display_links = ('database',)
    date_hierarchy = 'created'
    ordering = ('database', 'creator')
    list_filter = ("creator", "license")
    fieldsets = (
        (None, {
            'fields': (('database', 'description', 'license'), ('creator', 
                'modifier'), 'created', 'modified'),
            }
        ),
    )
    inlines = [GroupPermissionAdminInline,]


# Rendered as a charfield because loading large fields was killing the server.
class GeographyAdminForm(forms.ModelForm):
    geography = forms.CharField(widget=forms.TextInput)
    
    class Meta:
        model = Geography


class GeographyAdmin(admin.ModelAdmin):
    ordering = ('geography', 'alpha2')
    save_on_top = True
    search_fields = ('geography', 'description', 'alpha2', 'alpha3', 
        'imported_database_code',)
    list_display = ('geography', 'description', 'code', 'alpha2', 'alpha3', 
        'has_lat_long', 'process_count', 'weighting_count')
    list_filter = ('database',)
    fieldsets = (
        (None, {
            'fields': ('geography', 'description', 'longitude', 
            'latitude', 'database',),
            }
        ),
        ('Geo', {
            'classes': ('collapse',),
            'fields': ('area', 'pop_density_km', 'pop_total'),
            }
        ),
        ('Details', {
            'classes': ('collapse',),
            'fields': ('alpha2', 'alpha3', 'numeric', 
                'imported_database_code')
            }
        ),
    )

admin.site.register(Geography, GeographyAdmin)
