from django.db.models import signals
from brightway.apps.geo.models import GeographyDatabase, Geography

def create_geo_defaults(sender, **kwargs):
    wdb = GeographyDatabase.objects.get_or_create(database="World", 
        description="Default geography database for countries and regions." 
        )[0]
    Geography.objects.get_or_create(geography="Global", database=wdb, 
        description="Global geography")

# Django emits the post_syncdb signal after table creation, but before index
# creation. Unfortunately for us, geo columns in PostGIS are created via a 
# function, not a normal column description, so if this is called by
# post_syncdb it will fail with something like:
# django.db.utils.DatabaseError: column "poly_shape" of relation "geo_geography" does not exist
# because it doesn't (yet). See also:
# http://code.djangoproject.com/ticket/7561
# So, disable for now, and add manually to test cases, and to install docs.

# signals.post_syncdb.connect(create_geo_defaults, 
#     dispatch_uid='brightway.starter.create_geo_defaults')

