from django.db import transaction
from brightway.apps.geo.models import Geography, GeographyDatabase

def merge_geography_databases(first, second, updateable_fields=('imported_database_code',)):
    """Merge second into first, letting first take precedence. Merging done by name only."""
    transaction.enter_transaction_management()
    transaction.managed(True)

    assert isinstance(first, GeographyDatabase) and isinstance(second, GeographyDatabase)
    for old in Geography.objects.filter(database=second):
        if Geography.objects.filter(database=first, geography=old.geography).count() == 1:
            new = Geography.objects.get(database=first, geography=old.geography)
            print "Merging %s into %s" % (old, new)
            # Identical match. Ideal case
            merge_objects_to_new_geography(new, old, updateable_fields=updateable_fields)
            old.delete()
            
        elif Geography.objects.filter(database=first, geography=old.geography).count() > 1:
            print "Error with %s" % old
            # Error. error. error. abort!
            raise DuplicateGeographyError
        else:
            print "Putting %s into %s" % (old, first)
            # Easy
            old.database = first
            old.save()
    
    second.delete()
    transaction.commit()
    # transaction.rollback()
    transaction.leave_transaction_management()
