from django.contrib.auth.models import Group
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import FieldError

class GroupPermission(models.Model):
    """
    Allow the assignation of defined permission levels to a combination of a group and a model instance.
    """
    PERMISSION_CHOICES = (
    (0, _('open')), # Freely editable by anyone
    (1, _('usable')), # Freely readable, but not changable
    (2, _('restricted')), # Can't create variants or export
    (3, _('private')), # Private
    )

    group = models.ForeignKey(Group, verbose_name=_('group'))
    content_type = models.ForeignKey(ContentType, 
        verbose_name=_('content type'))
    object_id = models.PositiveIntegerField(_('object id'), db_index=True)
    content_object = generic.GenericForeignKey()
    permission = models.IntegerField(choices=PERMISSION_CHOICES, 
        verbose_name=_('permission level'))

    class Meta:
        unique_together = (('content_type', 'object_id', 'group'),)
        verbose_name = _('group permission')
        verbose_name_plural = _('group permissions')
        db_table = "gp_permissions" # Defining app_label breaks nose tests !?

    def __unicode__(self):
        return u'%s: %s: %s' % (self.content_object, self.group, self.permission)

    def save(self, *args, **kwargs): 
        #Django only does this validation in the admin
        if self.permission not in (0,1,2,3):
            raise FieldError, "Permission level must be between 0 and 3 (you gave %s)" % self.permission
        super(GroupPermission, self).save(*args, **kwargs)
