from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.models import User, Group
from django.core.exceptions import FieldError
from brightway.tests.extra_apps_test_case import ExtraAppsTestCase
from brightway.apps.group_permissions.tests.models import Foo, CreatedFoo
from brightway.apps.group_permissions.models import GroupPermission
from brightway.apps.group_permissions.utils import \
    get_permission_for_instance_and_user, create_update_permission

class GroupPermissionsTestCase(ExtraAppsTestCase):
    apps = ('apps.group_permissions.tests',)

    def setUp(self):
        super(GroupPermissionsTestCase, self).setUp()
        self.admins_group = Group.objects.get_or_create(name="admins")[0]
        self.everyone_group = Group.objects.get_or_create(name="everyone")[0]
        self.restricted_group = Group.objects.get_or_create(
            name="restricted group")[0]
        self.admin = User.objects.create_user("test admin", "a@a.com", 
            "admin")
        self.admin.is_superuser = True
        self.admin.is_staff = True
        self.admin.groups = (self.admins_group, self.everyone_group)
        self.admin.save()
        self.user = User.objects.create_user("test user", "a@a.com", "user")
        self.user.groups = (self.restricted_group, self.everyone_group)
        self.user.save()
        self.foo = Foo.objects.create(name="foo")
        self.bar = Foo.objects.create(name="bar")
        create_update_permission(self.foo, self.admins_group, 1)

    def test_retrieve_pmissions(self):
        self.assertEqual(get_permission_for_instance_and_user(self.foo, 
            self.admin), 0)
        self.assertEqual(get_permission_for_instance_and_user(self.foo, 
            self.user), 3)

    def test_everyone_permissions(self):
        # Although this is not enforced by the group_permissions application, 
        # every instance should have permissions assigned to the everyone 
        # group
        create_update_permission(self.foo, self.everyone_group, 2)
        self.assertEqual(get_permission_for_instance_and_user(self.foo, 
            self.user), 2)

    def test_unset_permissions(self):
        # Retrieving permissions where none were set
        self.assertEqual(get_permission_for_instance_and_user(self.bar, 
            self.admin), 0)
        self.assertEqual(get_permission_for_instance_and_user(self.bar, 
            self.user), 3)

    def test_with_creator_attribute(self):
        john = CreatedFoo.objects.create(name="foo", creator=self.user)
        self.assertEqual(get_permission_for_instance_and_user(john, self.user
            ), 0)

    def test_catch_incorrect_permissions(self):
        self.assertRaises(FieldError, create_update_permission, self.foo, 
            self.everyone_group, 4)

