from django.contrib import admin
from django.contrib.contenttypes import generic
from brightway.apps.impact_assessment.models import Method, Weighting
from brightway.apps.data_source.models import Source


class DataSourceInline(generic.GenericStackedInline):
    model = Source


class MethodAdmin(TreeEditor):
    save_on_top = True
    search_fields = ('method', 'description')
    list_display = ('method', 'weighting_count', 'parent', 
        'description')
    list_display_links = ('method',)
    fieldsets = (
        (None, {
            'fields': ('method', 'description', 'parent'),
            }
        ),
    )
    inlines = [DataSourceInline,]


class WeightingAdmin(admin.ModelAdmin):
    save_on_top = True
    search_fields = ('process__process',)
    list_display = ('process', 'method', 'geography', 'amount', 'description')
    list_filter = ('method',)
    raw_id_fields = ('process', 'geography')
    list_per_page = 25
    fieldsets = (
        (None, {
            'fields': ('process', 'method', 'geography', 'amount', 
                'description'),
            }
        ),
    )

admin.site.register(Weighting, WeightingAdmin)
admin.site.register(Method, MethodAdmin)

