from django.db import models
from django.core.exceptions import ObjectDoesNotExist

class MethodManager(models.Manager):
    def aggregate_submethods(self, method, recursive=True, overwrite=False):
        # Don't do anything if weightings already exist
        weighting_count = method.weighting_count()
        if weighting_count and not overwrite:
            return
        child_methods = method.get_children()

        # Only overwrite existing weightings if we have child methods
        if not child_methods.count():
            return
        method.weighting_set.all().delete()

        for sub_method in child_methods:
            if sub_method.get_children().count() and recursive:
                self.aggregate_submethods(sub_method, True, overwrite)
            for child_weighting in sub_method.weighting_set.all():
                try:
                    new_weighting = Weighting.objects.get(
                        method=method,
                        process=child_weighting.process,
                        geography=child_weighting.geography)
                    new_weighting.amount += child_weighting.amount
                    new_weighting.save(skip_datetime_update=True)

                except ObjectDoesNotExist:
                    Weighting.objects.create(
                        method=method,
                        process=child_weighting.process,
                        geography=child_weighting.geography,
                        amount=child_weighting.amount,
                        description=child_weighting.description,
                        other_info=child_weighting.other_info)

        # Update expiration timestamp
        method.save()

    def print_method_tree(self, method, level=0):
        print "\t"*level, "Weights: %i." % method.weighting_count(), method, \
            "(%i)" % method.id
        for child in method.get_children():
            self.print_method_tree(child, level = level + 1)
