# -*- coding: utf-8 -*-
import time
from brightway.apps.impact_assessment.models import Weighting, Method
from brightway.apps.geo.models import Geography
from brightway.tests.lca_test_case import LCATestCase
from brightway.apps.geo import WORLD


class ImpactAssessmentTestCase(LCATestCase):
    def test_add_gobal_geography_automatically(self):
        new_method = Method.objects.create(
            method="Another Method")
        w = self.create_weighting(method=new_method, 
            process=self.processes.nox, amount=1)
        self.assertEqual(WORLD(), w.geography)

    def test_geography_is_none(self):
        """Raise ValueError is Weighting(geography=None)."""
        new_method = Method.objects.create(
            method="Another Method")
        kwargs = {"process": self.processes.nox, 
                "method": new_method,
                "amount": 1,
                "geography": None}
        self.assertRaises(ValueError, Weighting.objects.create, **kwargs)

    def test_only_polygon_geographies_for_weightings(self):
        point = "POINT (2.1093749999157998 4.9158328011174000)"
        g = Geography.objects.create(shape=point, 
            database=self.geographies.worlddb)
        w = Weighting(process=self.processes.nox, 
            method=self.methods.human_health, 
            geography=g, 
            amount=1)
        self.assertRaises(AttributeError, w.save)

    def test_update_method_ancestors(self):
        old_value = self.methods.human_health.modified
        child = Method.objects.create(
            method="Child", 
            parent=self.methods.human_health)
        time.sleep(0.2)
        grandchild = Method.objects.create(
            method="Grandchild", 
            parent=child)
        hh = Method.objects.get(id=self.methods.human_health.id)
        self.assertTrue(hh.modified > old_value)