import math
from django.contrib.auth.models import Group
from django.db.models import Min, Manager, query, get_model, Q
from django.core.exceptions import ObjectDoesNotExist
from django.db import connection
from brightway.apps.group_permissions.utils import \
    filter_queryset_by_permission
from brightway.core.utils.database_math_test import DATABASE_SUPPORTS_MATH
# from brightway.apps.inventory.aggregation import CreatorMinCase

def groups(user):
    if user:
        return user.groups.all()
    else:
        return Group.objects.filter(name="Everyone")


class DatabaseQuerySet(query.QuerySet):
    def permissions(self, user=None):
        return self.filter(databasepermission__group__in=groups( 
            user)).annotate(permission=Min('databasepermission__permission'))

    def filter_by_permission(self, level, user=None):
        return self.filter(databasepermission__group__in=groups( 
            user)).annotate(permission=Min('databasepermission__permission') 
            ).filter(permission__lte=level)


class DatabaseManager(Manager):
    def get_query_set(self):
        model = get_model('inventory', 'Database')
        return DatabaseQuerySet(model)


class ExchangeManager(Manager):
    def fill_bounds(self, filter=None):
        # Overwrites all values, if filter is not provided
        from brightway.apps.inventory.models import Database
        # TODO: Work on providing queryset
        if DATABASE_SUPPORTS_MATH:
            lognormal_query = """update inventory_exchange set minimum = cached_amount / 2.71828182845904523536^sigma^2, maximum = cached_amount * 2.71828182845904523536^sigma^2 where minimum = 0 and maximum = 0 and uncertainty_type = %s""" % self.model.UNCERTAINTY_BOUNDED_LOGNORMAL
            normal_query = """update inventory_exchange set minimum = cached_amount - 2*sigma, maximum = cached_amount + 2*sigma where minimum = 0 and maximum = 0 and uncertainty_type = %s""" % self.model.UNCERTAINTY_BOUNDED_NORMAL
            cursor = connection.cursor()
            try:
                cursor.execute(lognormal_query)
                cursor.execute(normal_query)
                # TODO: Update references & children
                return
            except:
                pass
        # Do calculations in python
        # Lognormal
        self.model.UNCERTAINTY_BOUNDED_LOGNORMAL
        self.model.UNCERTAINTY_BOUNDED_NORMAL
        databases = Database.objects.filter(id__in=self.get_query_set(
            ).filter(minimum=0, maximum=0, uncertainty_type__in=(
            self.model.UNCERTAINTY_BOUNDED_LOGNORMAL, 
            self.model.UNCERTAINTY_BOUNDED_NORMAL)).values_list(
            'input__database', flat=True).distinct())
        # Refresh database metadata
        for database in databases:
            database.save()
        queryset = self.get_query_set().filter(uncertainty_type=\
            self.model.UNCERTAINTY_BOUNDED_LOGNORMAL, minimum=0, maximum=0)
        if queryset.count() > 500:
            # Use iterator
            queryset = queryset.iterator()
        for exchange in queryset:
            sigma_squared = math.exp(exchange.sigma)**2
            exchange.minimum = exchange.value / sigma_squared
            exchange.maximum = exchange.amount * sigma_squared
            exchange.save(skip_datetime_update=True)
        # Normal
        queryset = self.get_query_set().filter(uncertainty_type=\
            self.model.UNCERTAINTY_BOUNDED_NORMAL, minimum=0, maximum=0)
        if queryset.count() > 500:
            # Use iterator
            queryset = queryset.iterator()
        for exchange in queryset:
            exchange.minimum = exchange.value - 2*self.sigma
            exchange.maximum = exchange.amount + 2*self.sigma
            exchange.save(skip_datetime_update=True)

