# -*- coding: utf-8 -*-
from datetime import datetime, timedelta
from brightway.apps.inventory.models import *
from brightway.tests.lca_test_case import LCATestCase
from brightway.apps.inventory.errors import GeographyBiosphereError, \
    BiosphereFlowAlreadyExists

class BasicTestCase(LCATestCase):
    '''Test basic Process functionality.'''
    def test_db_setup(self):
        self.assertEquals(Process.objects.all().count(), 9)
        self.assertEquals(Exchange.objects.all().count(), 20)

    def test_date_overriding(self):
        now = datetime.now()
        e = Exchange.objects.filter(output=self.processes.coal)[0]
        m = Process.objects.get(id=self.processes.coal.id)
        d = Database.objects.get(id=self.databases.master.id)
        self.assertTrue(e.created - e.modified < timedelta(seconds=0.001))
        self.assertTrue(now > e.created)
        self.assertTrue(m.created - m.modified < timedelta(seconds=0.001))
        self.assertTrue(now > m.created)
        self.assertTrue(d.created - d.modified < timedelta(seconds=0.001))
        self.assertTrue(now > d.created)
        e.save()
        self.assertTrue(Database.objects.get(
            id=self.databases.master.id).modified > now)
        now = datetime.now()
        m.save()
        self.assertTrue(Database.objects.get(
            id=self.databases.master.id).modified > now)

    def test_mptt(self):
        # All basic operations are covered in MPTT tests. 
        # Call "python manage.py test --settings=library.mptt.tests.settings"
        self.assertEquals(Category.tree.root_nodes().count(), 6)
        self.assertEquals(Category.objects.get(category__iexact="energy" 
            ).get_children().count(), 1)
        self.assertEquals(Category.objects.get(category__iexact="energy" 
            ).get_siblings().count(), 5)
        self.assertEquals(Category.objects.get(category__iexact="energy" 
            ).get_descendants().count(), 1)
        self.assertEquals(Category.objects.get(category__iexact="coal" 
            ).get_ancestors().count(), 1)
        self.assertEquals(Category.objects.get(category__iexact="coal" 
            ).get_children().count(), 0)
        self.assertEquals(Category.objects.get(category__iexact="coal" 
            ).get_siblings().count(), 0)
        self.assertEquals(Category.objects.get(category__iexact="coal" 
            ).get_siblings(include_self=True).count(), 1)

    def test_biosphere_flows_cant_have_geo(self):
        nox = self.processes.nox
        nox.geography = self.geographies.europe
        self.assertRaises(GeographyBiosphereError, nox.save)

    def test_identical_biosphere_flows(self):
        attrs = {"name": "NOx", 
            "category": self.categories.airemissions,
            "geography": None,
            "unit": self.units.kg,
            "process_type": 2}
        self.assertRaises(BiosphereFlowAlreadyExists, self.create_process, 
            **attrs)

    def test_update_process_updates_database(self):
        now = self.databases.master.modified
        self.processes.nox.save()
        db = Database.objects.get(id=self.databases.master.id)
        self.assertTrue(now != db.modified)