# -*- coding: utf-8 -*-
from brightway.apps.inventory.models import Database, Category
from brightway.tests.lca_test_case import LCATestCase
from django_dag.errors import CircularReferenceError
from django_dag.models import Graph

class GraphTestCase(LCATestCase):
    '''Test interaction between Database graph and exchanges.'''
    def test_exchange_creates_database_link(self):
        """Test that new exchange creates link in database graph"""
        db = Database.objects.create(database="New one", 
            creator=self.users.admin)
        category = Category.objects.create(database=db, 
            category="Foo")
        process = self.create_process("New test process", category, 
            self.geographies.world, self.units.kg, database=db)
        exchange = self.create_exchange(self.processes.coal, process, 1)
        self.assertEqual(db.get_parents(), [self.databases.master,])
        self.assertEqual(db.get_ancestors(), [self.databases.master,])
        self.assertFalse(db.get_children())

    def test_process_inheritance_creates_database_link(self):
        db = Database.objects.create(database="New one", 
            creator=self.users.admin)
        category = Category.objects.create(database=db, 
            category="Foo")
        process = self.create_process("New test process", category, 
            self.geographies.world, self.units.kg, database=db, 
            parent=self.processes.electricity.coal)
        graph = Graph.objects.get(name="inheritance")
        self.assertEqual(db.get_parents(graph=graph), 
            [self.databases.master,])
        self.assertEqual(db.get_ancestors(graph=graph), 
            [self.databases.master,])
        self.assertFalse(db.get_children(graph=graph))

    def test_setup(self):
        """Test that link exists already in database graph"""
        self.assertFalse(self.databases.master.get_parents())
        self.assertEqual(self.databases.master.get_children(), 
            [self.databases.user,])
        self.assertEqual(self.databases.user.get_parents(), 
            [self.databases.master,])
        self.assertFalse(self.databases.user.get_children())

    def test_circular_reference(self):
        """Test that CircularReferenceError is raised"""
        db = Database.objects.create(database="New one", 
            creator=self.users.admin)
        db.add_parent(self.databases.master)
        self.assertRaises(CircularReferenceError, 
            self.databases.master.add_parent, db)

    def test_removing_database_dependency(self):
        """
Test that removing final exchange between databases deletes graph edge
        """
        pass

