from django.contrib import admin
from brightway.apps.parameter.models import GlobalVariable, LocalVariable

class GlobalVariableAdmin(admin.ModelAdmin):
    save_on_top = True
    search_fields = ('variable',)
    list_display = ('variable', 'database', 'creator')
    list_display_links = ('variable',)
    date_hierarchy = 'created'
    ordering = ('variable',)
    list_filter = ("database",)
    fieldsets = (
        (None, {
            'fields': (('variable', 'formula'), ('database', 'description',)),
            }
        ),
        ('Uncertainty', {
            'classes': ('collapse',),
            'fields': ('uncertainty_type', 'sigma', 'minimum', 'maximum'),
            }
        ),
        ('User & Creation', {
            'classes': ('collapse',),
            'fields': ('creator', 'modifier'),
            }
        ),
    )


class LocalVariableAdmin(admin.ModelAdmin):
    save_on_top = True
    search_fields = ('variable',)
    list_display = ('variable', 'process', 'creator')
    list_display_links = ('variable',)
    date_hierarchy = 'created'
    ordering = ('variable',)
    raw_id_fields = ('process',)
    fieldsets = (
        (None, {
            'fields': (('variable', 'formula'), ('process', 'description',)),
            }
        ),
        ('Uncertainty', {
            'classes': ('collapse',),
            'fields': ('uncertainty_type', 'sigma', 'minimum', 'maximum'),
            }
        ),
        ('User & Creation', {
            'classes': ('collapse',),
            'fields': ('creator', 'modifier'),
            }
        ),
    )


admin.site.register(LocalVariable, LocalVariableAdmin)
admin.site.register(GlobalVariable, GlobalVariableAdmin)

