import datetime
from brightway.core.utils.safe_hash import generate_safe_hash
from brightway.core.cache_interface import CacheInterface
from brightway.apps.parameter.errors import ScopeError


class ReferenceContext(object):
    """Provides a context for references in parameter graphs, including caching.
    
It has 3 objectives. First, we save lookups from checking formula scope so that 
we can re-use them when entering graph data. Second, we provide dictionary-like 
access so we can pass it simply to the evaluate method. Finally, we store values 
so that they can be pickled and retrieved quickly without hitting the database.

TODO: Caching."""
    def __init__(self, instance):
        self.instance = instance # Needed to create hash for caching
        self.hash = generate_safe_hash(instance._meta.db_table + str(
            instance.id))
        self.references = {} # Dictionary of keys by content type and id

    def keys(self):
        return self.references.keys()

    def values(self):
        return self.references.values()

    def __unicode__(self):
        if self.references:
            return u"ReferenceContext class with these references:" + \
                u"\n\t".join(["%s: %s" % (key, self.references[key]) for key in \
                self.references])
        else:
            return u"Empty ReferenceContext class with hash " + self.hash

    def __str__(self):
        return self.__unicode__()

    def __getitem__(self, key):
        try:
            return self.references[key]["amount"]
        except KeyError:
            raise ScopeError, "Reference %s is not in the current scope" % key
            