from pyparsing_mod import Word, nums, alphas, Combine, oneOf, \
    opAssoc, operatorPrecedence, hexnums, Optional, Or, alphanums, Suppress, \
    Forward, Group, ZeroOrMore, OneOrMore, Literal, ParseException, ParseResults

function_mapping = {
    "%": "*0.01",
    "ABS": "abs",
    "ACOS": "math.acos",
    "AND": "and",
    "ASIN": "math.asin",
    "ATAN": "math.atan",
    # ATAN2               math.atan2
    # AVERAGE             numpy.average       Is this the best approach?
    # CHOOSE == ? 
    "COS": "math.cos",
    # DEGREES 
    "EVEN": "functions.even",
    # EXACT 
    "EXP": "math.exp", 
    # FALSE == False 
    # IF 
    # INT 
    # LN 
    "LOG": "math.log",
    "LOG10": "math.log10", 
    # MAX 
    # MIN 
    "NOT": "not",
    "ODD": "functions.odd",
    # OR 
    "POWER": "math.pow", 
    # PRODUCT 
    # PROPER 
    # PV 
    # RADIANS 
    # RATE 
    # REPLACE 
    # REPT 
    # RIGHT 
    # ROUND 
    # ROWS 
    # SECOND 
    "SIN": "math.sin",
    # SLN 
    # SQRT 
    # STDEV 
    # STDEVP 
    # SUBSTITUTE 
    # SUM 
    "TAN": "math.tan", 
    # TIME 
    # TRUE 
}

LPAREN, RPAREN = map(Suppress, '()')

_hexStr = lambda n : Word(hexnums,exact=n)
reference = Combine(Literal("REF:<") + _hexStr(8) + ("-"+_hexStr(4))*3 + "-" + \
    _hexStr(12) + Literal(">"))
reference.setParseAction(lambda x: "ref:%s" % x[0][5:-1])

# Define numbers
plusorminus = Literal('+') | Literal('-')
digits = Word(nums) 
mantissa = Literal('.') + Optional(digits)
number = Combine( digits + Optional(mantissa) + Optional(oneOf('e E') + \
    Optional(plusorminus) + digits))
number.setParseAction(lambda x: str(float(x[0])))

normal_operands = oneOf('+ - * /')
comparison_operands = oneOf('= <> <= >= < >')
operands = normal_operands | comparison_operands

local_var = Combine(Word(alphas, min=1) + Optional(Word(alphanums)))
local_var.setParseAction(lambda x: "local:%s" % x[0])

global_var = Combine(Literal("global:") + Word(alphas, min=1) + \
    Optional(Word(alphanums)))

atoms = Forward()

# Generic parentheses
generic_sexp = Group(LPAREN + ZeroOrMore(atoms) + RPAREN)

# Functions
def replace_function_text(parsed):
    return [function_mapping[parsed[0][0]], parsed[0][1:]]

function = oneOf(" ".join(function_mapping.keys()))
function.setParseAction(lambda x: function_mapping[x[0]])

atoms << (number | operands | generic_sexp | function \
    | reference("references") | global_var("globals") | local_var("locals"))

parser = OneOrMore(atoms)