# -*- coding: utf-8 -*-
from brightway.apps.parameter.models import GlobalVariable
from brightway.tests.lca_test_case import LCATestCase

class FormulaMathTestCase(LCATestCase):
    """Test python math library functions within formulae."""
    def test_trig_functions(self):
        FORMULA_STRING = "COS(0) + SIN(3.1415926535897931/2) + TAN(0)"
        gv = GlobalVariable(variable="test", database=self.databases.master, 
            formula=FORMULA_STRING, creator=self.users.admin)
        gv.save(atomic=False)
        self.assertEquals(gv.amount, 2)

    def test_logarithmic_functions(self):
        gv = GlobalVariable(variable="test", database=self.databases.master, 
            formula="LOG(1) + 2 * LOG10(10)", creator=self.users.admin)
        gv.save(atomic=False)
        self.assertEquals(gv.amount, 2)

    def test_zero_division(self):
        gv = GlobalVariable(variable="test", database=self.databases.master, 
            formula="1", creator=self.users.admin)
        gv.formula = "1/0"
        self.assertRaises(ZeroDivisionError, gv.save, atomic=False)

    def test_overflow(self):
        # Currently let this get raised as an error.
        return
        # gv = GlobalVariable.objects.create(variable="test", 
        #     database=self.databases.master, formula="1",
        #     creator=self.users.admin)
        # gv.formula = "10e300 * 10e300"
        # self.assertRaises(OverflowError, gv.save)
