# -*- coding: utf-8 -*-
from django.core.exceptions import ObjectDoesNotExist
from brightway.apps.inventory.models import Process, Exchange, Database
from brightway.apps.parameter.models import GlobalVariable, LocalVariable
from brightway.tests.lca_test_case import LCATestCase
from django_dag.models import Graph, Node, Edge, TransitiveClosure

class ParameterTestCase(LCATestCase):
    """Test basic functionality of formulas, and proper error handling."""
    def create_variable(self, database=None, variable="Var1", amount=1.0, 
            creator=None, formula=None):
        if not database:
            database = self.databases.master
        if not creator:
            creator = self.users.admin
        gv = GlobalVariable(database=database, amount=amount,
            variable=variable, creator=creator, formula=formula)
        gv.save(atomic=False)
        return gv

    def test_create_single_reference(self):
        var1 = self.create_variable()
        var2 = self.create_variable(amount=None, formula="global:Var1 * 2", 
            variable="Var2")
        self.assertEqual(Edge.objects.filter(node_from=var1.node, 
            node_to=var2.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_from=var1.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_to=var2.node).count(), 1)
        self.assertEqual(var1.amount, 1)
        self.assertEqual(var2.amount, 2)

    def test_create_chained_references(self):
        var1 = self.create_variable()
        var2 = self.create_variable(amount=None, formula="global:Var1 * 2", 
            variable="Var2")
        var3 = self.create_variable(amount=None, formula="global:Var2 * 2", 
            variable="Var3")
        self.assertEqual(Edge.objects.filter(node_from=var1.node, 
            node_to=var2.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_from=var2.node, 
            node_to=var3.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_from=var1.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_to=var2.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_from=var2.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_to=var3.node).count(), 1)
        self.assertEqual(var1.amount, 1)
        self.assertEqual(var2.amount, 2)
        self.assertEqual(var3.amount, 4)

    # def test_create_double_incoming_references(self):
    #     var1 = self.create_variable()
    #     formula1 = self._create_formula(name="Formula with references", 
    #         formula="3 * var:%i" % var1.id)
    #     formula2 = self._create_formula(name="Formula with references 2", 
    #             formula="2 * var:%i" % var1.id)
    #     self.assertEquals(formula1.amount, 3)
    #     self.assertEquals(formula2.amount, 2)
    #     self.assertEquals(var1.references.count(), 0)
    #     self.assertEquals(formula1.references.count(), 1)
    #     self.assertEquals(formula2.references.count(), 1)
    # 
    # def test_create_double_outgoing_references(self):
    #     var1 = self.create_variable()
    #     var2 = self.create_variable(variable="Tester 2", amount=2.0)
    #     formula1 = self._create_formula(name="Formula with references", 
    #         formula="2 * var:%i + 3 * var:%i" % (var1.id, var2.id))
    #     self.assertEquals(var1.amount, 1)
    #     self.assertEquals(var2.amount, 2)
    #     self.assertEquals(formula1.cached_amount, 8)
    #     self.assertEquals(formula1.amount, 8)
    #     self.assertEquals(var1.references.count(), 0)
    #     self.assertEquals(var2.references.count(), 0)
    #     self.assertEquals(formula1.references.count(), 2)
    # 
    # def test_attributes(self):
    #     pass
    # 
    # def test_parent_formula_changed(self):
    #     formula1 = Formula.objects.create(name="Test plain formula", 
    #         formula="10", creator=self.users.admin)
    #     var1 = self.create_variable()
    #     var1.amount = formula1
    #     var1.save()
    #     formula2 = Formula.objects.create(name="Formula with references", 
    #             formula="2 * var:%i" % var1.id, creator=self.users.admin)
    #     var2 = self.create_variable(formula=formula2, variable="Tester 2", 
    #         amount=None)
    #     self.assertEquals(formula2.amount, 20)
    #     self.assertEquals(var2.amount, 20)
    #     formula1.formula = "5"
    #     formula1.save()
    #     self.assertEquals(formula1.amount, 5)
    #     self.assertEquals(Variable.objects.get(id=var1.id).amount, 5)
    #     self.assertEquals(Formula.objects.get(id=formula2.id).amount, 10)
    #     self.assertEquals(Variable.objects.get(id=var2.id).amount, 10)
    # 
    # def test_mixed_parameters(self):
    #     formula1 = Formula.objects.create(name="Test plain formula", 
    #         formula="10", creator=self.users.admin)
    #     var1 = self.create_variable(amount=None, formula=formula1)
    #     formula2 = Formula.objects.create(name="F2", 
    #         formula="2*var:%i" % var1.id, creator=self.users.admin)
    #     formula3 = Formula.objects.create(name="F3", 
    #         formula="3 * var:%i" % var1.id, creator=self.users.admin)
    #     var2 = self.create_variable(amount=None, formula=formula2, 
    #         variable="2")
    #     var3 = self.create_variable(amount=None, formula=formula3, 
    #         variable="3")
    #     formula4 = Formula.objects.create(name="F4", 
    #         formula="(var:%i + var:%i)/40" % (var2.id, var3.id), 
    #         creator=self.users.admin)
    #     var4 = self.create_variable(amount=None, formula=formula4, 
    #         variable="4")
    #     formula5 = Formula.objects.create(name="F5", 
    #         formula="var:%i + var:%i" % (var2.id, var4.id), 
    #         creator=self.users.admin)
    #     self.assertEquals(formula5.amount, 21.25)
    # 
    # # Test formula evaluates as NaN
    # 
    # def test_reference_deleted(self):
    #     var1 = self.create_variable(variable="Should disappear")
    #     var2 = self.create_variable(variable="Should stay", amount=2.0)
    #     formula1 = self._create_formula(name="Formula with references", 
    #         formula="2 * var:%i + 3 * var:%i" % (var1.id, var2.id))
    #     self.assertEquals(var1.amount, 1)
    #     self.assertEquals(var2.amount, 2)
    #     formula2 = Formula.objects.get(id=formula1.id)
    #     formula2.formula="3 * var:%i" % var2.id
    #     formula2.save()
    #     var3 = Variable.objects.get(id=var1.id)
    #     var4 = Variable.objects.get(id=var2.id)
    #     self.assertEquals(formula2.amount, 6)
    #     self.assertEquals(var3.referenced_by.count(), 0)
    #     self.assertEquals(var4.referenced_by.count(), 1)
    #     self.assertEquals(formula2.references.count(), 1)
    #     
    # def test_conflicting_names(self):
    #     self.create_variable()
    #     self.assertRaises(IntegrityError, self.create_variable)
    # 
