# -*- coding: utf-8 -*-
from brightway.tests.lca_test_case import LCATestCase
from brightway.apps.inventory.models import Process, Exchange, Database
from brightway.apps.parameter.models import GlobalVariable, LocalVariable
from brightway.apps.parameter.errors import IllegalLocalVariable, \
    DuplicateGlobalVariable
from django_dag.models import Graph, Node, Edge, TransitiveClosure

class GlobalVariableTestCase(LCATestCase):
    def create_variable(self, database=None, variable="Var1", amount=1.0, 
            creator=None, formula=None):
        if not database:
            database = self.databases.master
        if not creator:
            creator = self.users.admin
        gv = GlobalVariable(database=database, amount=amount,
            variable=variable, creator=creator, formula=formula)
        gv.save(atomic=False)
        return gv

    def test_create_single_reference(self):
        var1 = self.create_variable()
        var2 = self.create_variable(formula="global:Var1 * 2", 
            variable="Var2")
        self.assertEqual(Edge.objects.filter(node_from=var1.node, 
            node_to=var2.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_from=var1.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_to=var2.node).count(), 1)
        self.assertEqual(var1.amount, 1)
        self.assertEqual(var2.amount, 2)

    def test_create_chained_references(self):
        var1 = self.create_variable()
        var2 = self.create_variable(formula="global:Var1 * 2",
            variable="Var2")
        var3 = self.create_variable(formula="global:Var2 * 2", 
            variable="Var3")
        self.assertEqual(Edge.objects.filter(node_from=var1.node, 
            node_to=var2.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_from=var2.node, 
            node_to=var3.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_from=var1.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_to=var2.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_from=var2.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_to=var3.node).count(), 1)
        self.assertEqual(var1.amount, 1)
        self.assertEqual(var2.amount, 2)
        self.assertEqual(var3.amount, 4)

    def test_duplicate_global_variable(self):
        self.create_variable()
        self.assertRaises(DuplicateGlobalVariable, self.create_variable)


class LocalVariableTestCase(LCATestCase):
    def create_variable(self, variable="Var1", amount=1.0, creator=None, 
            formula=None, process=None):
        if not process:
            process = self.processes.steel
        if not creator:
            creator = self.users.admin
        return LocalVariable.objects.create(process=process, amount=amount,
            variable=variable, creator=creator, formula=formula)

    def test_create_single_reference(self):
        var1 = self.create_variable()
        var2 = self.create_variable(formula="Var1 * 2", variable="Var2")
        self.assertEqual(Edge.objects.filter(node_from=var1.node, 
            node_to=var2.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_from=var1.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_to=var2.node).count(), 1)
        self.assertEqual(var1.amount, 1)
        self.assertEqual(var2.amount, 2)

    def test_create_chained_references(self):
        var1 = self.create_variable()
        var2 = self.create_variable(formula="Var1 * 2", variable="Var2")
        var3 = self.create_variable(formula="Var2 * 2", variable="Var3")
        self.assertEqual(Edge.objects.filter(node_from=var1.node, 
            node_to=var2.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_from=var2.node, 
            node_to=var3.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_from=var1.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_to=var2.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_from=var2.node).count(), 1)
        self.assertEqual(Edge.objects.filter(node_to=var3.node).count(), 1)
        self.assertEqual(var1.amount, 1)
        self.assertEqual(var2.amount, 2)
        self.assertEqual(var3.amount, 4)
