from django.contrib import admin
from brightway.apps.unit.models import *

class UnitTypeAdmin(admin.ModelAdmin):
    save_on_top = True
    save_as = True
    search_fields = ('unittype', 'description',)
    list_display = ('unittype', 'description', 'unit_count', 'process_count')
    fieldsets = (
        (None, {
            'fields': (('unittype', 'description',),),
            }
        ),
    )


class UnitAdmin(admin.ModelAdmin):
    save_on_top = True
    save_as = True
    search_fields = ('unit', 'abbreviation', 'description',)
    list_display = ('unit', 'abbreviation', 'unittype', 'description', 
        'process_count')
    prepopulate_from = {"abbreviation": ("unit",),}
    list_filter = ("unittype",)
    fieldsets = (
        (None, {
            'fields': (('unit', 'abbreviation',), 'description', 'unittype'),
            'description': "Abbreviation is for the full unit, e.g. kg for kilogram."
            }
        ),
    )

admin.site.register(UnitType, UnitTypeAdmin)
admin.site.register(Unit, UnitAdmin)

