class Container(object):
    """Container class for attributes"""
    pass

def create_defaults():
    """Create some defaults for Brightway install."""
    from django.contrib.auth.models import Group, User
    from django_dag.models import Graph
    from brightway.apps.geo.models import GeographyDatabase, Geography
    from brightway.core.utils.new_secret_key import get_new_secret_key

    # Create parameter graph
    pg, created = Graph.objects.get_or_create(name="parameter")
    if not created:
        return
    Graph.objects.get_or_create(name="inheritance")
    Graph.objects.get_or_create(name="database")
    gdb = GeographyDatabase.objects.create(database="Global", 
        description="Geography database for Global geography. No other " +\
        "geographies are allowed.")
    wdb = GeographyDatabase.objects.create(database="World", 
        description="Default geography database for countries and regions.")
    # Called world because GLOBAL is too close to python reserved word
    Geography.objects.create(geography="Global", database=gdb, 
        description="Global geography. Used as default if no location " +\
        "information is available.", imported_database_code="GLO")
    # Create Everyone group and Importer user. Could exist already is 
    # superuser is created at first syncdb call.
    everyone = Group.objects.get_or_create(name="Everyone")[0]
    # Set Importer to random password
    User.objects.create(username="Importer", password=get_new_secret_key())
    public = User.objects.create(username="public", 
        password=get_new_secret_key())
    public.groups.add(everyone)
