-- Create an in-memory table with the following columns that is recursively
-- populated as the tree is followed upwards.
with recursive exchange_recursive (
    -- The ID of the current level exchange (fixed to exchange)
    exchange_id, 
    -- The ID of the current process's parent (changes with recursion)
    intermediate_parent_id, 
    -- The id of the exchange output (fixed to exchange, comes from parent, etc.)
    exchange_output_id,
    -- The name of the exchange output (fixed to exchange, comes from parent, etc.)
    exchange_output_name,
    -- The input process id
    input_id, 
    -- The input process name
    input_name,
    -- The output process id
    output_id, 
    -- The output process name
    output_name,
    -- The exchange amount
    amount, 
    -- The level of recursion (changes with recursion)
    level,
    -- The output process type (doesn't change)
    output_type,
    -- The input process type (fixed to exchange)
    input_type,
    -- Exchange uncertainty: Sigma (fixed to exchange)
    sigma,
    -- Exchange uncertainty: Type (fixed to exchange)
    uncertainty_type,
    -- Exchange uncertainty: Minimum (fixed to exchange)
    minimum,
    -- Exchange uncertainty: Maximum (fixed to exchange)
    maximum,
    -- The following are text strings, as this information is used in the UI
    geography, category, database, unit
    ) as (
    select e.id, starting_table.parent_id, starting_table.id, starting_table.process, input.id, input.process, starting_table.id, starting_table.process, e.amount, 0, starting_table.process_type, input.process_type, e.sigma, e.uncertainty_type, e.minimum, e.maximum, geography.geography, category.compoundname, database.database, unit.unit
        from inventory_process starting_table
        left join inventory_exchange e
            on starting_table.id = e.output_id
        left join inventory_process input
            on input.id = e.input_id
        left join inventory_database database
            on database.id = input.database_id
        left join unit_unit unit
            on unit.id = input.unit_id
        left join inventory_category category
            on category.id = input.category_id
        left join geo_geography geography
            on geography.id = input.geography_id
        where starting_table.id = %s
    union all
    select e.id, recursed_table.parent_id, recursed_table.id, recursed_table.process, input.id, input.process, exchange_recursive.output_id, exchange_recursive.output_name, e.amount, exchange_recursive.level + 1, exchange_recursive.output_type, input.process_type, e.sigma, e.uncertainty_type, e.minimum, e.maximum, geography.geography, category.compoundname, database.database, unit.unit
        from exchange_recursive, inventory_process recursed_table
        left join inventory_exchange e
            on recursed_table.id = e.output_id
        left join inventory_process input
            on input.id = e.input_id
        left join inventory_database database
            on database.id = input.database_id
        left join unit_unit unit
            on unit.id = input.unit_id
        left join inventory_category category
            on category.id = input.category_id
        left join geo_geography geography
            on geography.id = input.geography_id
        where recursed_table.id = exchange_recursive.intermediate_parent_id
        )
select distinct on (input_id, output_id) *
    from exchange_recursive
    where input_id is not null
    and output_id is not null
    and level > 0
    order by input_id, output_id, level asc