from brightway.apps.inventory.models import Process, Exchange
from stats_toolkit.uncertainty_choices import uncertainty_choices

class InheritedExchangeProxy(object):
    """A proxy object for a virtual exchange, inherited from a parent process."""
    def __init__(self, row_dict, extended=False):
        self.amount = row_dict['amount']
        self.input_id = row_dict['input_id']
        self.output_id = row_dict['output_id']
        self.parent_exchange_id = row_dict['exchange_id']
        self.parent_process_id = row_dict['exchange_output_id']
        self.inherited = row_dict['level'] > 0
        self.level = row_dict['level']
        self.uncertainty_type_id = row_dict['uncertainty_type']
        self.sigma = row_dict['sigma']
        self.minimum = row_dict['minimum']
        self.maximum = row_dict['maximum']
        if extended:
            self.input_name = row_dict.get('input_name')
            self.input_type = row_dict.get('input_type')
            self.output_name = row_dict.get('output_name')
            self.output_type = row_dict.get('output_type')
            self.geography = row_dict.get('geography')
            self.category = row_dict.get('category')
            self.database = row_dict.get('database')
            self.unit = row_dict.get('unit')

    def __unicode__(self):
        if self.inherited:
            return u"Proxy for inherited exchange from %s to %s" % (
                self.input, self.output)
        else:
            return u"Proxy for exchange from %s to %s" % (
                self.input, self.output)

    def __repr__(self):
        return self.__unicode__()

    def create_exchange(self):
        raise NotImplementedError

    @property
    def uncertainty_type(self):
        return uncertainty_choices[self.uncertainty_type_id]

    @property
    def parent_exchange(self):
        if not self.inherited:
            return False
        return Exchange.objects.get(id=self.exchange_id)

    @property
    def parent_process(self):
        if not self.inherited:
            return False
        return Process.objects.get(id=self.parent_process_id)

    @property
    def input(self):
        return Process.objects.get(id=self.input_id)

    @property
    def output(self):
        return Process.objects.get(id=self.output_id)

