-- Create an in-memory table with the following columns that is recursively
-- populated as the tree is followed upwards.
with recursive weighting_recursive (
    -- The ID of the current level weighting (fixed to weighting)
    weighting_id, 
    -- The ID of the child flow (doesn't change)
    process_id, 
    -- The ID of the current flows's parent (changes with recursion)
    intermediate_parent_id, 
    -- The weighting amount (fixed to weighting)
    amount, 
    -- The level of recursion (changes with recursion)
    level,
    -- Exchange uncertainty: Sigma (fixed to exchange)
    sigma,
    -- Exchange uncertainty: Type (fixed to weighting)
    uncertainty_type,
    -- Exchange uncertainty: Minimum (fixed to weighting)
    minimum,
    -- Exchange uncertainty: Maximum (fixed to weighting)
    maximum
    ) as (
    select w.id, starting_table.id, starting_table.parent_id, w.amount, 0, w.sigma, w.uncertainty_type, w.minimum, w.maximum
        from inventory_process starting_table
        -- The key is a left join, as some biosphere flows may not have 
        -- weightings. We need the empty row with the parent id for the 
        -- recursive part, so we can pull in the weightings of the parent.
        left join ia_weighting w
            on starting_table.id = w.process_id
        where starting_table.process_type in %s
        and (w.method_id = %s or w.method_id is null)
        and (w.geography_id = %s or w.geography_id is null)
    union all
    -- This is the recursive part; the first section is the initial rows.
    -- These are the recursive rows. Level is the level up the tree.
    select w.id, weighting_recursive.process_id, recursed_table.parent_id,  w.amount, weighting_recursive.level + 1, w.sigma, w.uncertainty_type, w.minimum, w.maximum
        from weighting_recursive, inventory_process recursed_table
        left join ia_weighting w
            on recursed_table.id = w.process_id
        -- Recursive link by parent id
        where recursed_table.id = weighting_recursive.intermediate_parent_id
        and (w.method_id = %s or w.method_id is null)
        and (w.geography_id = %s or w.geography_id is null)
    )
select distinct on (process_id) process_id, weighting_id, intermediate_parent_id, amount, level, sigma, uncertainty_type, minimum, maximum 
    from weighting_recursive
    where weighting_id is not null
    -- Ascending order so it picks lowest level first
    order by process_id, level asc