from scipy.sparse import lil_matrix
from brightway.core.lca.lca import LCA
from brightway.apps.inventory.models import Process
from brightway.core.array_proxies import CompressedSparseMatrixProxy


class FrozenLCA(LCA):
    """Operates on frozen processes, where inventory is already calculated."""
    def __init__(self, *args, **kwargs):
        super(FrozenLCA, self).__init__(*args, **kwargs)
        self.check_process_is_frozen()

    def check_process_is_frozen(self):
        if not self.process_queryset.count() == 1:
            raise ValueError("Frozen LCA can only operate on one process")
        if not self.process_queryset[0].process_type == Process.FROZEN_TYPE:
            raise ValueError(
                "Frozen LCA can only operate on frozen processes")

    def lci(self):
        obj = self.process_queryset[0]
        self.dicts.technosphere = {obj.id: 0}
        self.technosphere_count = 1
        self.dicts.biosphere = dict([(key, index) for index, key in \
            enumerate(obj.frozen_data)])
        self.biosphere_count = len(self.dicts.biosphere)
        frozen = lil_matrix((self.biosphere_count, 1))
        for key, value in obj.frozen_data.iteritems():
            frozen[self.dicts.biosphere[key], 0] = value
        self.calculated_biosphere = CompressedSparseMatrixProxy(
            frozen.tocsr(), self.dicts.biosphere, self.dicts.technosphere)
        self.lci_finished = True

    def redo_lci(self, *args, **kwargs):
        raise NotImplementedError

