# -*- coding: utf-8 -*
from __future__ import division
import logging
from scipy.sparse.linalg import iterative
from scipy.sparse.linalg import spsolve
from brightway.core.lca.monte_carlo import MonteCarloLCA
from brightway.core.logging_decorators import timelogger


class IterativeMonteCarloLCA(MonteCarloLCA):
    """Monte Carlo, using iterative solvers instead of matrix decomposition."""
    def __init__(self, process, iterative_method=iterative.cgs, **kwargs):
        self.parse_iterative_method(iterative_method)
        super(IterativeMonteCarloLCA, self).__init__(process, **kwargs)

    def parse_iterative_method(self, iterative_method):
        if not iterative_method in (iterative.bicg, iterative.bicgstab, 
                iterative.cg, iterative.cgs, iterative.gmres, iterative.qmr):
            raise ValueError, "Iterative solver must be an instance of scipy.linalg.iterative solvers"
        self.iterative_method = iterative_method

    @timelogger(logging.INFO)
    def solve_linear_system(self, a, b): # Can pass a, b
        """Use iterative solvers instead of decomposition."""
        # TODO: If over a certain number of iterations: Calculate and use preconditioner for technological matrix
        # Can use approximate inverse of seed technology matrix
        # M : {sparse matrix, dense matrix, LinearOperator}
        #     Preconditioner for A.  The preconditioner should approximate the
        #     inverse of A.  Effective preconditioning dramatically improves the
        #     rate of convergence, which implies that fewer iterations are needed
        #     to reach a given error tolerance.
        if not hasattr(self, "_seed"):
            self._seed = spsolve(a,b)
            return self._seed
        else:
            s, status = self.iterative_method(a, b, x0=self._seed)
            if status != 0:
                if self.logging and status > 0:
                    self.logger.warning("Iterative solver didn't converge after %i iterations" % status)
                elif self.logging:
                    self.logger.crtiical("Iterative solver fed invalid inputs")
                return spsolve(a,b)
            return s
