# -*- coding: utf-8 -*
from __future__ import division
import time
from brightway.core.lca.monte_carlo import MonteCarloLCA
from brightway.core.lca.iterative_monte_carlo import IterativeMonteCarloLCA
from stats_toolkit.random import LatinHypercubeRNG

class LatinHypercubicLCA(MonteCarloLCA):
    """.. centered:: Monte Carlo analysis using Latin Hypercubic sampling.

Has generally the same form, inputs, and outputs as MonteCarloLCA. `samples` defines the number of samples to draw from (default is 10). The higher the number of samples, to more accurately the distribution is modeled, but the slower it converges. As samples goes to infinity, this becomes the simple Monte Carlo LCA.

TODO: Caching.
    """
    def __init__(self, process, samples=10, **kwargs):
        super(LatinHypercubicLCA, self).__init__(process, **kwargs)
        self.rng = LatinHypercubeRNG
        self.samples = samples
        self.rng_kwargs = {'samples': self.samples}


class IterativeLatinHypercubicLCA(IterativeMonteCarloLCA):
    """.. centered:: Iterative Monte Carlo analysis using Latin Hypercubic sampling.

Strict subclass of :ref:`latin-hypercubic-rng` and :ref:`iterative_monte_carlo`.
    """
    def __init__(self, process, samples=10, **kwargs):
        super(IterativeLatinHypercubicLCA, self).__init__(process, **kwargs)
        self.rng = LatinHypercubeRNG
        self.samples = samples
        self.rng_kwargs = {'samples': self.samples}
