from brightway.core.lca.lca import LCA
from scipy import sparse
import operator

class PowerSeriesLCA(LCA):
    def __init__(self, process, iterations=25, *args, **kwargs):
        self.iterations = iterations
        super(PowerSeriesLCA, self).__init__(process, *args, **kwargs)

    def solve_linear_system(self, *args): # Can pass a, b
        """Solve the system using power series expansion instead of the pseudoinverse."""
        if args:
            if not len(args) == 2:
                raise LCASetupError, "Called solve method with invalid inputs"
            a, b = args
        else:
            a, b = self._technology_matrix.data, self._answer_array.data
    
        dimensions = a.shape[0]
        identity = sparse.spdiags([1], [0], dimensions, dimensions)
        a_prime = identity - a
        power_series_list = [a_prime**x for x in range(1,self.iterations+1)]
        # We use reduce to sum a list of csr matrices
        power_series = reduce(operator.add, power_series_list)
        # Return to proper form
        power_series = identity + power_series
        return power_series*b