# -*- coding: utf-8 -*-
import datetime
from brightway.core.lca.lca import LCA
from scipy import sparse, linalg, matrix
try:
    from scipy.sparse.linalg import spsolve
except ImportError:
    # Scipy < 0.7
    from scipy.linsolve import spsolve
from numpy import array, allclose, identity, zeros
from brightway.apps.inventory.models import Database, Exchange, Process
from brightway.tests.lca_test_case import LCATestCase
from brightway.core.logs.loggers import lca_logger as logger

class BasicTestCase(LCATestCase):
    def test_lca_calculations(self):
        #Get in correct form
        technosphere_reference_matrix = sparse.lil_matrix(identity(
            self.technological_matrix.shape[0]) - self.technological_matrix)
        steel_reference_demand_array = zeros(
            technosphere_reference_matrix.shape[0])
        steel_reference_demand_array[self.processes.technological.index(
            self.processes.steel)] = 1 #1 kg of steel
        steel_reference_answer = spsolve(
            technosphere_reference_matrix.tocsr(), 
            steel_reference_demand_array)

        L = LCA(self.processes.steel, read_from_cache=False, 
            write_to_cache=False, **self.redirect_kwargs)
        L.lci()
        print "Starting output"
        print L.database
        print L.technosphere_count
        print L.technosphere_matrix.data.toarray()
        print technosphere_reference_matrix
        print self.technological_matrix
        self.assertEquals(L.technosphere_count, 
            technosphere_reference_matrix.shape[0])
        self.assertTrue(allclose(L.supply_array.data, steel_reference_answer))
        self.assertTrue(allclose(L.technosphere_matrix.data.toarray(), 
            technosphere_reference_matrix.toarray()))

