# -*- coding: utf-8 -*-
from brightway.core.lca.lca import LCA
from brightway.apps.inventory.models import Database, Exchange, Process
from brightway.tests.lca_test_case import LCATestCase

class InheritanceTestCase(LCATestCase):
    """Test that LCA correctly implements exchange inheritance"""
    def setUp(self):
        super(InheritanceTestCase, self).setUp()
        self.fusion = self.create_process("Fusion", 
            self.categories.electricity, self.geographies.world, 
            self.units.kwh, parent=self.processes.electricity.wind)
        self.fusion.attributes['dirty'] = False
        # Delete PM emission
        self.create_exchange(self.processes.pm, self.fusion, 0, 
            uncertainty_type=0)
        # Add input of electricity from wind
        self.create_exchange(self.processes.electricity.wind, self.fusion, 
            0.1)
        # Change input of electricity from mix
        self.create_exchange(self.processes.electricity.mix, self.fusion, 0.7)

    def test_technosphere_matrix_construction(self):
        l = LCA(self.fusion)
        l.lci()
        self.assertEqual(l.technosphere_matrix.shape, (6,6))
        self.assertEqual(l.technosphere_matrix.nnz, 20)

    def test_biosphere_matrix_construction(self):
        l = LCA(self.fusion)
        l.lci()
        self.assertEqual(l.biosphere_matrix.shape, (3,6))
        self.assertEqual(l.biosphere_matrix.nnz, 8)

    def test_exchanges_array_dimensions(self):
        """Test that the exchanges array has the correct dimensions."""
        l = LCA(self.fusion)
        l.lci()
        self.assertEqual(l.exchanges_array.shape, (22,))
        self.assertEqual(len(l.exchanges_array.dtype.names), 13)

    def test_exchanges_array_with_inherited_processes(self):
        """Tests that the exchanges array has the correct processes and values."""
        pass
