# -*- coding: utf-8 -*-
import datetime
import sys
from scipy import sparse, linalg, matrix
from numpy import *

from brightway.core.lca.lca import LCA
from brightway.core.lca.monte_carlo import MonteCarloLCA, LCAOperationalError
from brightway.core.lca.latin_hypercubic import LatinHypercubicLCA
from brightway.apps.inventory.models import Database, Exchange, Process
from brightway.tests.lca_test_case import LCATestCase

class MonteCarloTestCase(LCATestCase):
    def test_calcualtion_finishes(self):
        m = MonteCarloLCA(self.processes.steel, iterations=10, 
            method=self.methods.human_health, **self.redirect_kwargs)
        self.assertFalse(m.mc_completed)
        m.calculate()
        self.assertTrue(m.mc_completed)

    def test_mc_errors(self):
        m = MonteCarloLCA(self.processes.steel, iterations=100, 
            method=self.methods.human_health, **self.redirect_kwargs)
        self.assertRaises(LCAOperationalError, m.lci)
        self.assertRaises(LCAOperationalError, m.lcia)

    def test_mc_lcia_results(self):
        return
        L = LCA(self.processes.steel, method=self.methods.human_health, 
            **self.redirect_kwargs)
        L.lci()
        L.lcia()
        m = MonteCarloLCA(self.processes.steel, iterations=200, 
            method=self.methods.human_health, **self.redirect_kwargs)
        m.calculate()
        mc_results = m.mc.biosphere_results.sum(axis=0)
        # print mc_results.shape
        # print average(mc_results)
        # print l.weighted_biosphere[0].sum()
        # print "Technology matrix"
        # print l.technology_matrix.toarray()
        # print m.mc.technology_matrix_instance.toarray()
        # print "Weighting matrix"
        # print l.weighting_matrix[0].data
        # print m.weighting_array
        # print "Emissions matrix"
        # print l.emission_matrix.toarray()
        # print m.mc.emission_matrix_instance.toarray()
        # print "Supply array"
        # print l.supply_array
        # print m.mc.supply_results[:,0]
        # print "Top processes"
        # print l.top_processes()
        # print "Top emissions"
        # print l.top_emissions()
        # print "CTV"
        # print m.ctv_top()
        # print "Top processes:"
        # print l.top_processes()
        # print "MC results:", average(mc_results)
        # print "LCA result:", l.weighted_biosphere[0].sum()
        # print "MC Matrix:"
        # print m.mc.technology_matrix_instance.todense()
        # print "LCA Matrix:"
        # print l.technology_matrix.todense()
        # print "MC Emission matrix instance:"
        # print m.mc.emission_matrix_instance.todense()
        # print "LCA emission matrix:"
        # print l.emission_matrix.todense()
        # print "Tech array:"
        # print m.mc.technology_array
        # print "Technology positions"
        # print m.mc.technology_positions
        # print "LCA supply array:"
        # print l.supply_array.data
        # print "MC Supply array:"
        # print average(m.mc.supply_results, axis=1)
        self.assertTrue(abs(average(mc_results)-L.weighted_biosphere[0].sum()
            ) < 0.1 * L.weighted_biosphere[0].sum())

    def test_mc_bounded_distributions(self):
        L = LCA(self.processes.steel, method=self.methods.human_health, 
            **self.redirect_kwargs)
        L.lci()
        L.lcia()
        m = MonteCarloLCA(self.processes.steel, iterations=200, 
            method=self.methods.human_health, **self.redirect_kwargs)
        m.calculate()
        mc_results = m.mc.biosphere_results.sum(axis=0)
        self.assertTrue(abs(average(mc_results)-L.weighted_biosphere[0].sum()
            ) < 0.1 * L.weighted_biosphere[0].sum())

class LatinHypercubicTestCase(LCATestCase):
    def test_compare(self):
        m = MonteCarloLCA(self.processes.steel, iterations=200, 
            method=self.methods.human_health, **self.redirect_kwargs)
        m.calculate()
        mc_results = m.mc.biosphere_results.sum(axis=0)
        lh = LatinHypercubicLCA(self.processes.steel, iterations=200, 
            method=self.methods.human_health, **self.redirect_kwargs)
        lh.calculate()
        lh_results = lh.mc.biosphere_results.sum(axis=0)
        print "Monte Carlo histogram:"
        print histogram(mc_results)
        print "Average:", average(mc_results)
        print "Monte Carlo vector:", m.mc.technosphere_generator.iterate()
        print "Monte Carlo params:"
        print "Latin Hypercube histogram:"
        print histogram(lh_results)
        print "Average:", average(lh_results)
        print "LH vector:", lh.mc.technosphere_generator.iterate()
        # print "Hypercube:", lh.mc.technology_generator.hypercube
        print "LH Params:"
