import time
import logging
from functools import wraps

def timelogger(level=logging.INFO):
    def decorator(func):
        message = func.func_name
        @wraps(func)
        def wrapper(*args, **kwargs):
            t = time.time()
            ret_val = func(*args, **kwargs)
            other = args[0] # Other is self
            if other.logging:
                other.logger.log(level, message+": %.3g" % (time.time() - t))
            return ret_val
        return wrapper
    return decorator

def catch_ioerror(func):
    @wraps(func)
    def with_logging(*args, **kwargs):
        other = args[0]
        try:
            return func(*args, **kwargs)
        except IOError, err:
            if other.logging:
                other.logger.log(logging.ERROR, err)
            return False

    return with_logging


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    example_logger = logging.getLogger('package1.module1')

    class C(object):
        """Simple example class to test logging decorator"""
        def __init__(self, name=None):
            self.name=name
            self.logging = True
            self.logger = example_logger
        
        @timelogger(logging.INFO)
        def f1(self):
            time.sleep(1.1)
        
        @timelogger(logging.DEBUG)
        def f2(self, foo):
            print "Got argument %s" % foo
            time.sleep(0.9)
        
    c = C("Example class")
    c.f1()
    c.f2("bar")