import os
from django.conf import settings
from django.db import connections, transaction

def always_rollback(using=None):
    """Always rollback a transaction, regardless of result"""
    def entering(using):
        transaction.enter_transaction_management(using=using)
        transaction.managed(True, using=using)

    def exiting(exc_value, using):
        transaction.rollback(using=using)
        transaction.leave_transaction_management(using=using)

    return transaction._transaction_func(entering, exiting, using)

def sql(obj, params=(), using="default", as_dict=False):
    """
Execute arbitrary SQL, passed either as a filepath or a string.

Filenames should be relative to BRIGHTWAY_ROOT_PATH.

Outputs a cursor object. It is up to the consumer to fetchone(), fetchall(), etc.

.. rubric:: Inputs

* obj: String. Either a filepath or a string for the SQL to execute.
* params: Tuple. Parameters for SQL query
* using: String. Database to use (useful only with multiple databases)

.. rubruc:: Outputs

A python DBABI cursor object.
    """
    # TODO: Pass stringIO object as well?
    try:
        filepath = os.path.join(settings.BRIGHTWAY_ROOT_PATH, obj)
        query = open(filepath, "r").read()
    except IOError:
        # Assume query is a string
        query = obj
    cursor = connections[using].cursor()
    cursor.execute(query, params)
    if as_dict:
        return dict_cursor(cursor)
    else:
        return cursor

def dict_cursor(cursor):
    """Turn SQL tuples into dictionaries. Returns generator."""
    description = [x[0] for x in cursor.description]
    for row in cursor:
        yield dict(zip(description, row))

def sql_one_row(obj, params=(), using="default"):
    cursor = sql(obj, params, using, True)
    results = cursor.next()
    return results
