### Note: This won't work for Attributes. It fails for non-abstract model inheritance.
from django.db.models import AutoField
from django.utils.translation import ugettext
from django.db import transaction
from brightway.apps.inventory.models import Database, Category, Process, \
    Exchange

# TODO: This is creating multiple copies of stuff

class CopyMachine(object):
    def __init__(self, obj, name=None, cascade=False, action=True, 
            commit=True):
        self.obj = obj
        self.mapping_dict = {}
        if action:
            self.start_transaction()
            try:
                if isinstance(obj, Database):
                    self.copy_database(self.obj, name=name, cascade=cascade)
                elif isinstance(obj, Category):
                    self.copy_category(self.obj, cascade=cascade)

                if commit:
                    self.commit_transaction()
                else:
                    self.rollback_transaction()
            except:
                self.rollback_transaction()
                raise

    def start_transaction(self):
        transaction.enter_transaction_management()
        transaction.managed(True)

    def rollback_transaction(self):
        transaction.rollback()

    def commit_transaction(self):
        transaction.commit()

    def copy_model_instance(self, obj, name=False, name_field=False, 
            save=True, **kwargs):
        obj = self._copy_model_instance(obj)
        # UUID could be an AutoField, but I don't know how that works with
        # metaclasses
        if hasattr(obj, 'uuid'):
            obj.uuid = None

        for key in kwargs:
            setattr(obj, key, kwargs[key])

        # Defaults to "foo copy"
        if name != u"!skip":
            # Returns "database" for Database, for example
            if not name_field:
                name_field = obj._model_string.lower()

            if not name:
                # Not correct order for all languages, but OK
                name = getattr(obj, name_field) + u" " + ugettext(u"copy")

            setattr(obj, name_field, name)

        if save:
            obj.save()

        return obj

    def _copy_model_instance(self, obj):
        # http://www.djangosnippets.org/snippets/1040/
        initial = dict([(f.name, getattr(obj, f.name))
                        for f in obj._meta.fields
                        if not isinstance(f, AutoField) and \
                        not f in obj._meta.parents.values()])
        return obj.__class__(**initial)

    def copy_database(self, database, name=None, cascade=False, **kwargs):
        new = self.copy_model_instance(database, name=name, **kwargs)
        if not 'database' in self.mapping_dict:
            self.mapping_dict['database'] = {}
        self.mapping_dict['database'][database.uuid] = new.uuid
        self.copy_group_permissions_for_obj(database, new)

        if cascade:
            for category_parent in Category.objects.filter(database=database, 
                    parent__isnull=True):
                self.copy_category_tree(category_parent, database=new)

            for process in Process.objects.filter(database=database):
                category = Category.objects.get(uuid=self.mapping_dict[
                    'category'][process.category.uuid])
                self.copy_process(process, name="!skip", database=new,
                    category=category)

            for exchange in Exchange.objects.filter( 
                    output__database=database):
                # Switch input if needed
                if exchange.input.uuid in self.mapping_dict['process']:
                    input = Process.objects.get(uuid=self.mapping_dict[
                        'process'][exchange.input.uuid])
                else:
                    input = exchange.input
                output = Process.objects.get(uuid=self.mapping_dict[
                    'process'][exchange.output.uuid])
                self.copy_exchange(exchange, name="!skip", input=input,
                    output=output)

        return new

    def copy_category_tree(self, category_parent, name=None, cascade=False, 
            **kwargs):
        for category in category_parent.get_descendants(
                include_self=True):
            # Adjust parent category is needed
            if hasattr(category.parent, "uuid") and \
                    category.parent.uuid in self.mapping_dict[
                    'category']:
                parent = Category.objects.get(uuid=self.mapping_dict[
                    'category'][category.parent.uuid])
                self.copy_category(category, name="!skip", 
                    parent=parent, **kwargs)
            else:
                self.copy_category(category, name="!skip", **kwargs)

        if cascade:
            pass

        return None # Creates multiple categories

    def copy_category(self, category, name=None, cascade=False, **kwargs):
        # Use copy_category_tree to copy entire tree
        new = self.copy_model_instance(category, name=name, **kwargs)
        if not 'category' in self.mapping_dict:
            self.mapping_dict['category'] = {}
        self.mapping_dict['category'][category.uuid] = new.uuid
        self.copy_attributes_for_obj(category, new)

        if cascade:
            for process in Process.objects.filter(category=category):
                category = Category.objects.get(uuid=self.mapping_dict[
                    'category'][process.category.uuid])
                self.copy_process(process, name="!skip", category=new)

            for exchange in Exchange.objects.filter( 
                    output__category=category):
                # Switch input if needed
                if exchange.input.uuid in self.mapping_dict['process']:
                    input = Process.objects.get(uuid=self.mapping_dict[
                        'process'][exchange.input.uuid])
                else:
                    input = exchange.input
                output = Process.objects.get(uuid=self.mapping_dict[
                    'process'][exchange.output.uuid])
                self.copy_exchange(exchange, name="!skip", input=input,
                    output=output)

        return new

    def copy_process(self, process, name=None, cascade=False, **kwargs):
        new = self.copy_model_instance(process, name=name, **kwargs)
        if not 'process' in self.mapping_dict:
            self.mapping_dict['process'] = {}
        self.mapping_dict['process'][process.uuid] = new.uuid
        self.copy_attributes_for_obj(process, new)

        if cascade:
            for exchange in Exchange.objects.filter(output=process):
                # Switch input if needed
                if exchange.input.uuid in self.mapping_dict['process']:
                    input = Process.objects.get(uuid=self.mapping_dict[
                        'process'][exchange.input.uuid])
                else:
                    input = exchange.input
                output = Process.objects.get(uuid=self.mapping_dict[
                    'process'][exchange.output.uuid])
                self.copy_exchange(exchange, name="!skip", input=input,
                    output=output)

        return new

    def copy_exchange(self, exchange, name=None, cascade=False, **kwargs):
        new = self.copy_model_instance(exchange, name=name, **kwargs)
        if not 'exchange' in self.mapping_dict:
            self.mapping_dict['exchange'] = {}
        self.mapping_dict['exchange'][exchange.uuid] = new.uuid
        self.copy_attributes_for_obj(exchange, new)

        return new

    def copy_group_permissions_for_obj(self, old, new):
        # TODO: Make work for models without generic FK
        auto_created_permissions = new.group_permission_set.all(
            ).values_list("group", "object_id")
        for gp in old.group_permission_set.all():
            # Overwrite auto-created permissions if needed
            if (gp.group.id, new.id) in auto_created_permissions:
                gp_new = new.group_permission_set.get(group__id=gp.group.id,
                    object_id=new.id)
                gp_new.permission = gp.permission
                gp_new.save()
            else:
                gp_new = self.copy_model_instance(gp, name="!skip", save=False)
                gp_new.object_id = new.id
                gp_new.save()

    def copy_attributes_for_obj(self, old, new):
        # TODO: Make work for models without attribute manager
        return
        for att in old.objects.all():
            att_new = self.copy_model_instance(att, name="!skip", save=False)
            att_new.object_id = new.id
            att_new.save()

