import csv
# from cStringIO import StringIO 
from django.contrib.contenttypes.models import ContentType
from django.db.models.query import QuerySet
from django.db.models import Model
from django.http import HttpResponse 
# from brightway.io.create_zipfile import zipfile_from_buffers
from brightway.io.file_sender import FileSender
# Optional Excel writers
try:
    import xlwt
except ImportError:
    xlwt = False

class GenericModelExporter(object):
    """
Class for exporting models to various formats. Accepts an instance or queryset.
    """
    def __init__(self, obj, format="csv", filename=None, zipped=False, in_memory=False, **kwargs):
        super(GenericModelExporter, self).__init__()
        if obj and isinstance(obj, QuerySet):
            self.obj = obj
            self.model = obj.model
        elif obj and hasattr(obj, "_meta"):
            self.obj = [obj,] # Make iterable
            self.model = obj.__class__
        else:
            raise AttributeError, "Can't understand object"
        self.format = format
        self.zipped = zipped
        self.in_memory = in_memory
        self.modelname = self.model._meta.verbose_name_plural.__unicode__()
        if filename:
            self.filename = filename
        else:
            self.filename = u"%s_%s" % (
                self.modelname, "_".join([str(x.id) for x in self.obj]))
        self.kwargs = kwargs

    if xlwt:
        excel_styles = {'datetime': xlwt.easyxf(num_format_str='yyyy-mm-dd hh:mm:ss'),
                  'date': xlwt.easyxf(num_format_str='yyyy-mm-dd'),
                  'time': xlwt.easyxf(num_format_str='hh:mm:ss'),
                  'default': xlwt.Style.default_style}
    else:
        excel_styles = None

    def export(self):
        """Choose correct exporter based on format"""
        # if self._use_stringio:
        #     self.file = StringIO()
        # else:
        #     self.file = open(self.filename, "w")
        self.file_sender = FileSender(filename=self.filename, zip_=self.zipped, in_memory=self.in_memory, content_type=self.format.lower())
        
        if self.format.lower() == "csv":
            self._export_plain(separator=",")
        elif self.format.lower() == "tsv":
            self._export_plain(separator="\t")
        elif self.format.lower() == "excel":
            self._export_excel()
        elif self.format.lower() == "gdocs":
            pass
        elif self.format.lower() == "html":
            pass
        else:
            raise AttributeError, "Format not recognized"
        # return self.file

    def _export_plain(self, separator=","):
        """Export to plain text using standard library CSV"""
        # use a StringIO buffer rather than opening a file 
        if "dialect" in self.kwargs:
            w = csv.writer(self.file_sender.file, **self.kwargs)
        elif separator == ",":
            w = csv.writer(self.file_sender.file, dialect=csv.excel)
        elif separator == "\t":
            w = csv.writer(self.file_sender.file, dialect=csv.excel_tab)
        else:
            raise
        # Header
        w.writerow(self.utf8_encoder([self.modelname,] + [x.id for x in 
            self.obj]))
        for field in self.model._meta.fields:
            if field.name == "id":
                continue
            w.writerow(self.utf8_encoder([field.name,] + [getattr(x, 
                field.name) for x in self.obj]))

    def utf8_encoder(self, data):
        def _encode(element):
            if hasattr(element, "encode"):
                return element.encode("utf-8")
            else:
                return element
        if isinstance(data, (str, unicode)):
            return _encode(data)
        else:
            return [_encode(element) for element in data]

    def _export_excel(self):
        if not xlwt:
            return self._export_plain()
        
        book = xlwt.Workbook(encoding='utf-8')
        sheet = book.add_sheet('Brightway Export')

        for rowx, row in enumerate(data):
            for colx, value in enumerate(row):
                if isinstance(value, datetime.datetime):
                    cell_style = self.excel_styles['datetime']
                elif isinstance(value, datetime.date):
                    cell_style = self.excel_styles['date']
                elif isinstance(value, datetime.time):
                    cell_style = self.excel_styles['time']
                else:
                    cell_style = styles['default']
                sheet.write(rowx, colx, value, style=cell_style)
        book.save(self.file)
        
        
    # def admin_export_xls(request, app, model):
    #     mc = ContentType.objects.get(app_label=app, model=model).model_class()
    #     wb = xlwt.Workbook()
    #     ws = wb.add_sheet(unicode(mc._meta.verbose_name_plural))
    #     for i, f in enumerate(mc._meta.fields):
    #         ws.write(0,i, f.name)
    #     qs = mc.objects.all()
    #     for ri, row in enumerate(qs):
    #         for ci, f in enumerate(mc._meta.fields):
    #             ws.write(ri+1, ci, unicode(getattr(row, f.name)))
    #     fd, fn = tempfile.mkstemp()
    #     os.close(fd)
    #     wb.save(fn)
    #     fh = open(fn, 'rb')
    #     resp = fh.read()
    #     fh.close()
    #     response = HttpResponse(resp, mimetype='application/ms-excel')
    #     response['Content-Disposition'] = 'attachment; filename=%s.xls' % \
    #           (unicode(mc._meta.verbose_name_plural),)
    #     return response

