try:
    import amara
except ImportError:
    amara = False
import codecs
from xml.dom import XML_NAMESPACE as XML_NS
from django.contrib.gis.shortcuts import render_to_kmz, render_to_kml
from django.db.models import Min, Max
from django.template.loader import render_to_string
from brightway.apps.inventory.models import Process
from brightway.apps.impact_assessment.models import Method, Weighting
from brightway.apps.geo.models import Geography
from brightway.io.file_sender import FileSender
from brightway.io.errors import ConfigurationError
from brightway.core.utils.file_utils import check_if_expired
from brightway.core.utils.safe_hash import generate_safe_hash
from brightway.core.graphics.nonlinear_colormap import NonlinearColormap
from brightway.core.graphics.convert import rgb_to_kml
from brightway.apps.geo.geometry_choices import POINT, LINESTRING, POLYGON, \
    MULTIPOLYGON, LINEARRING

def render_geography_to_kml(objs, compress=False, 
        template="kml/geography_queryset.html", filename=None, 
        use_file_sender=True):
    "View that renders a Geography object or queryset to a KML file."
    if (hasattr(objs, "count") and objs.count() == 0):
        # Nothing to do
        return

    if isinstance(objs, Geography):
        template = "kml/geography.html"
        filename = filename if filename else objs.geography
    elif objs.count() == 1:
        template = "kml/geography.html"
        objs = objs[0]
        filename = filename if filename else objs.geography
    else:
        filename = filename if filename else "Geography collection"

    if use_file_sender:
        file_sender = FileSender()

        if compress:
            file_sender = FileSender(filename, zip_=True, content_type="kmz")
            file_sender.add_zip_file(filename + ".kml", 
                render_to_string(template, {"g": objs}))
            return file_sender.send()
        else:
            file_sender = FileSender(filename, content_type="kml")
            file_sender.file.write(render_to_string(template, {"g": objs}))
            return file_sender.send()
    else:
        f = open(filename + ".kml", "w")
        f.write(render_to_string(template, {"g": objs}))
        f.close()
