import amara
from brightway.apps.geo.models import Geography

class MetadataExporter(object):
    '''Export metadata for easy input into Brightway'''
    def __init__(self, arg):
        super(MetadataExporter, self).__init__()
        self.arg = arg
        self.optional_geography_attributes = (
            u'numeric',
            u'description',
            u'area',
            u'pop_density_km',
            u'pop_total',
            u'alpha2',
            u'alpha3',
            u'imported_database_code')

    def locations():
        doc = amara.create_document(u'geography')
        for database in self.geography_databases:
            database_element = doc.xml_create_element(u'locationDatabase', content=database.database,
                attributes={
                    'uuid': database.uuid,
                    'description': database.description,
                    })
            if database.parents:
                parents_element = database_element.xml_create_element(u'parents')
                for parent_database in database.parents:
                    parents_element.xml_append(parents_element.xml_create_element(parent_database.uuid))
                database_element.xml_append(parents_element)
            for geography in Geography.objects.filter(database__in=self.geography_databases):
                element = doc.locations.xml_create_element(u'location',
                    content=geography.geography,
                    attributes={ # These attributes are required
                        u'longitude': geography.longitude,
                        u'latitude': geography.latitude,
                        u'uuid': geography.uuid})
                for attr in optional_attributes:
                    if hasattr(geography, attr):
                        element.xml_set_attribute(attr, getattr(geography, attr))
                if geography.shape:
                    element.xml_set_attribute(u'wkt', geography.shape.wkt)
                    element.xml_set_attribute(u'srid', geography.shape.srid)
                database_element.xml_append(element)
            doc.locationDatabase.xml_append(database_element)