import csv 
from django.db.models.query import QuerySet
from brightway.apps.inventory.models import Process, Exchange
from brightway.io.export.generic_model_export import GenericModelExporter

# Optional Excel writers
try:
    import xlwt
except ImportError:
    xlwt = False
try:
    import pyexcelerator
except ImportError:
    pyexcelerator = False

class ProcessExporter(GenericModelExporter):
    """
Class for exporting Processes to various formats. Accepts an instance or queryset.
    """
    ATTRIBUTES = (
        (u"Name", "process"),
        (u"Database", "database"),
        (u"Category", "category"),
        (u"Description", "description"),
        (u"Geography", "geography"),
        (u"Unit", "unit"),
        (u"Process Type", "process_type"),
        (u"Selection Type", "selection_type"),
        (u"CAS Number", "cas_number"),
    )

    def _export_plain(self, separator=","):
        """Export to plain text using standard library CSV"""
        if "dialect" in self.kwargs:
            w = csv.writer(self.file_sender.file, **self.kwargs)
        elif separator == ",":
            w = csv.writer(self.file_sender.file, dialect=csv.excel)
        elif separator == "\t":
            w = csv.writer(self.file_sender.file, dialect=csv.excel_tab)
        else:
            raise
        
        for obj in self.obj:
            w.writerow(self.utf8_encoder([u"Process details",]))
            for label, att in self.ATTRIBUTES:
                w.writerow(self.utf8_encoder([label, u"%s" % getattr(obj, att)]))
            w.writerow(["", ""])
            
            w.writerow(self.utf8_encoder([u"Inputs",]))
            w.writerow(self.utf8_encoder([u"Input", u"Amount", u"Unit", u"Description", u"Uncertainty Type", u"Sigma", "Minimum", u"Maximum"]))
            for exchange in Exchange.objects.filter(output=obj):
                w.writerow(self.utf8_encoder(["%s" % exchange.input, "%.3g" % exchange.amount, u"%s" % exchange.input.unit, u"%s" % exchange.description, "%s" % exchange.get_uncertainty_type_display(), "%.3g" % exchange.sigma, "%.3g" % exchange.minimum, "%.3g" % exchange.maximum]))
            w.writerow(["", ""])

            # w.writerow(self.utf8_encoder([u"Outputs",]))
            # w.writerow(self.utf8_encoder([u"Output", u"Amount", u"Unit"]))
            # for exchange in Exchange.objects.filter(input=obj):
            #     w.writerow(self.utf8_encoder(["%s" % exchange.output, "%.2e" % exchange.amount, u"%s" % exchange.input.unit]))
            # w.writerow(["", ""])
            w.writerow(["", ""])
