import datetime
from brightway.apps.inventory.models import Process
from brightway.io.imprt.import_base import ImportBase
from brightway.apps.impact_assessment.models import Weighting, Method
from brightway.apps.unit.models import Unit
from brightway.apps.geo import WORLD

class ImpactAssessmentBase(ImportBase):
    """Base class for import impact assessment methods."""
    def create_method(self, method, description='', other_info='', 
            parent=None, uuid=None, unit=None):
        if not unit:
            unit = Unit.objects.get(unit="kilogram")
        method = Method(method=method,
            description=description, 
            other_info=other_info,
            parent=parent,
            uuid=uuid, 
            unit=unit,
            modified=datetime.datetime.now())
        method.save(skip_datetime_update=True)
        return method

    def create_weighting(self, method, process, amount, geography=None, 
            uncertainty_type=0, sigma=0, creator=None,
            description='', other_info=''):
        if not creator:
            creator = self.user
        if not geography:
            geography = WORLD()
        weighting = Weighting(method=method,
            process=process,
            amount=amount,
            geography=geography,
            uncertainty_type=uncertainty_type,
            sigma=sigma,
            description=description,
            creator=creator,
            other_info=other_info)
        weighting.save(skip_datetime_update=True, atomic=False) 
        return weighting

    def create_weighting_for_flows(self, method, emission_label, sphere, 
            amount, geography=None, database=None, **kwargs):
        if not database:
            database = self.database
        assert sphere in ("air", "water", "soil")
        flows = Process.objects.filter(process_type__in=(1,2), 
            process=emission_label, database=database, 
            category__parent__category=sphere)
        if not flows.count():
            return 0
        ids = [self.create_weighting(method, flow, amount, 
            geography=geography, **kwargs).id for flow in flows]
        return Weighting.objects.filter(id__in=ids)