#!/usr/bin/env python

# For this script to work we must set the Django settings file
# as an environment setting before importing LayerMapping

# Alternatively you can place 
# export DJANGO_SETTINGS_MODULE=settings
# in your .bash_profile
#
# or paste this code into a $ manage.py shell

import os
# os.environ['DJANGO_SETTINGS_MODULE'] = 'brightway.settings'
from psycopg2 import IntegrityError

from django.contrib.gis.utils import add_postgis_srs
from django.contrib.gis.geos.collections import MultiPolygon
from django.conf import settings
from brightway.io.imprt.layer_mapping import SuperLayerMapping
from brightway.apps.geo.models import Geography, GeographySimplified

try:
    add_postgis_srs(900913)
except IntegrityError:
    print "The Google Spherical Mercator projection, or a projection with srid 900913, already exists, skipping insert"

WORLD_SHP = os.path.join(settings.BRIGHTWAY_ROOT_PATH, "io/data/geography/world/TM_WORLD_BORDERS-0.3.shp")

mapping = {'alpha2': 'ISO2', 'alpha3': 'ISO3', 'area': 'AREA', 'pop_total': 'POP2005', 'longitude': 'LON', 'latitude': 'LAT', 'geography': 'NAME', 'shape': 'MULTIPOLYGON', 'numeric': 'UN'}

layer = SuperLayerMapping(Geography,
                      WORLD_SHP,
                      mapping,
                      transform=False,
                      encoding='iso-8859-1')
                    
layer.save(verbose=True,strict=True,progress=True)


print 'Fixing invalid polygons...'
num = len(Geography.objects.extra(where=['NOT ST_IsValid(shape)']))
for item in Geography.objects.extra(where=['NOT ST_IsValid(shape)']):
    corrected = item.shape.buffer(0)
    item.shape = corrected
    item.save()

print 'Adding simplified shapes'
for g in Geography.objects.all():
    try:
        h = g.simplified
    except:
        # Not found
        gs = GeographySimplified()
        gs.geography = g
        ss = g.shape.simplify(0.05, preserve_topology=True)
        if not isinstance(ss, MultiPolygon):
            ss = MultiPolygon(ss)
        gs.simplified_shape=ss
        gs.save()

num = len(Geography.objects.extra(where=['NOT ST_IsValid(shape)']))
if num:
  print '%s invalid polygons remain' % num
  print Geography.objects.extra(where=['NOT ST_IsValid(shape)'])