from django.test import TestCase
from brightway.core import create_defaults
from brightway.apps.geo import WORLD
from brightway.apps.inventory.models import Category, Process, Exchange, \
    Database, DatabasePermission
from brightway.apps.impact_assessment.models import Weighting

class BaseTestCase(TestCase):
    """Base test case that creates Brightway defaults."""
    def setUp(self):
        super(BaseTestCase, self).setUp()
        # Need to reset cache, otherwise keeps old database and id values
        create_defaults()
        WORLD(reset=True)

    def create_process(self, name, category, geography=None, unit=None, 
            parent=None, creator=None, database=None, process_type=0):
        creator = creator if creator else self._default_user
        database = database if database else self._default_db
        geography = geography if geography else self._world
        unit = unit if unit else self._kg
        return Process.objects.create(process=name, 
                            category=category,
                            parent=parent,
                            geography=geography,
                            unit=unit,
                            database=database, 
                            creator=creator,
                            modifier=creator,
                            process_type=process_type)

    def create_exchange(self, input, output, amount, input_group=5, 
            output_group=2, creator=None, uncertainty_type=2, sigma=0.2):
        creator = creator if creator else self._default_user
        e = Exchange(output=output,
            input=input,
            amount=amount,
            uncertainty_type=uncertainty_type,
            input_group=input_group,
            output_group=output_group,
            creator=creator,
            modifier=creator,
            sigma=sigma)
        e.save(atomic=False, skip_datetime_update=True)

    def create_weighting(self, method, process, amount, geography=None, 
            creator=None, uncertainty_type=2, sigma=0.2):
        if not creator:
            creator = self._default_user
        if not geography:
            geography = self._world
        return Weighting.objects.create(method=method,
                            process=process,
                            amount=amount,
                            geography=geography,
                            uncertainty_type=uncertainty_type,
                            creator=creator,
                            modifier=creator,
                            sigma=sigma)

    def create_category(self, category, parent=None, database=None):
        if not database:
            database=self._default_db
        return Category.objects.create(category=category,
            parent=parent, database=database)

    def create_database(self, database, goal="", scope="", description="", 
            database_type=2, license=None, creator=None, managers=(), 
            users=(), viewers=()):
        creator = creator if creator else self._default_user
        db = Database.objects.create(
            database=database, 
            database_type=database_type, 
            license=license, 
            goal=goal, 
            scope=scope, 
            description=description, 
            creator=creator)
        for group in managers:
            DatabasePermission.objects.create(
                group=group, 
                database=db,
                permission=0)
        for group in users:
            DatabasePermission.objects.create(
                group=group, 
                database=db,
                permission=1)
        for group in viewers:
            DatabasePermission.objects.create(
                group=group, 
                database=db,
                permission=2)
        return db

