import os
import json
import bz2
from django.core import serializers
from django.conf import settings
from django_dag.models import Graph, Node, Edge
from brightway.apps.geo.models import Geography, GeographyDatabase

def dump_temp_files():
    JSONSerializer = serializers.get_serializer("json")
    json_serializer = JSONSerializer()

    plain_apps = ("auth", "impact_assessment", "inventory", "license", 
        "parameter", "unit")
    # Uses generic foreign keys -> natural keys under serialization
    nk_apps = ("bw_attributes", "data_source", "group_permissions")
    # TransitiveClosure created by triggers
    nk_models = (Graph, Node, Edge) 
    # Must be called manually for proper serialization of geo objects
    geo_models = (Geography, GeographyDatabase) 

    to_concatenate = []
    # counter = 1

    for app in plain_apps:
        filepath = os.path.join(settings.BRIGHTWAY_ROOT_PATH, 
            "core/tests/fixtures/temp/", "%s.json" % app)
        to_concatenate.append(filepath)
        command = "./manage.py dumpdata %s > %s" % (app, filepath)
        print "Running:", command
        os.system(command)

    for app in nk_apps:
        filepath = os.path.join(settings.BRIGHTWAY_ROOT_PATH, 
            "core/tests/fixtures/temp/", "%s.json" % app)
        to_concatenate.append(filepath)
        command = "./manage.py dumpdata --natural %s > %s" % (app, filepath)
        print "Running:", command
        os.system(command)

    for model in nk_models:
        filepath = os.path.join(settings.BRIGHTWAY_ROOT_PATH, 
            "core/tests/fixtures/temp/", "%s.json" % \
            model._meta.module_name)
        to_concatenate.append(filepath)
        print "Writing:", filepath
        with open(filepath, "w") as f:
            json_serializer.serialize(model.objects.all(), stream=f, 
                use_natural_keys=True)

    for model in geo_models:
        filepath = os.path.join(settings.BRIGHTWAY_ROOT_PATH, 
            "core/tests/fixtures/temp/", "%s.json" % \
            model._meta.module_name)
        to_concatenate.append(filepath)
        print "Writing:", filepath

        # Need to use "eager" to avoid deferred geographies -> no serializing
        if hasattr(model, "eager"):
            qs = model.eager.all()
        else:
            qs = model.objects.all()

        with open(filepath, "w") as f:
            json_serializer.serialize(qs, stream=f)

    return to_concatenate

def concatenate(files=None):
    if not files:
        temp_dir = os.path.join(settings.BRIGHTWAY_ROOT_PATH, 
            "core/tests/fixtures/temp/")
        files = [os.path.join(temp_dir, x) for x in filter(lambda x: x[-5:
            ] == ".json", os.listdir(temp_dir))]
    data = []
    for filepath in sorted(files):
        with open(filepath, "r") as f:
            decoded = json.load(f)
            if isinstance(decoded, (list, tuple)):
                data.extend(decoded)
    with open(os.path.join(settings.BRIGHTWAY_ROOT_PATH, 
            "core/tests/fixtures/open-io.json.bz2"), "wb") as f:
        f.write(bz2.compress(json.dumps(data, indent=2)))


if __name__ == "__main__":
    files = dump_temp_files()
    concatenate(files)